import os.path
from abc import ABC, abstractmethod
import json
from typing import Optional


class FileLoader(ABC):

    @abstractmethod
    def load(self, filepath, raise_error: bool = True) -> Optional[dict]:
        pass

    @staticmethod
    @abstractmethod
    def for_type() -> str:
        pass


class JsonFileLoader(FileLoader):

    @staticmethod
    def for_type() -> str:
        return "json"

    def load(self, filepath, raise_error: bool = True) -> Optional[dict]:

        if not os.path.exists(filepath) and not raise_error:
            return {}

        with open(filepath, "rb") as file:
            data = json.load(file)

            if type(data) != dict:
                raise ValueError("Expected the data loaded from the file to be in dictionary format (json object).")

            return data


class LoaderStrategy:
    JSON = JsonFileLoader
    # Preparation to support other file formats in the future. ex: yml




from siba import SIBA_SETTINGS
from siba.parser import Parsable, ParsingRule


class PluralizedParsable(Parsable):

    def __init__(self, data: dict):
        self.data = data

    def parse(self, key: str, *args, **kwargs) -> str:
        pluralization_count = kwargs.get("p_count", 1)
        if pluralization_count == 0:
            return self.data["none"]
        if pluralization_count == 1:
            return self.data["one"]

        pluralization_settings = SIBA_SETTINGS.get("pluralization", {})

        if (
            pluralization_settings.get("some_enabled", False)
            and 1 < pluralization_count < pluralization_settings.get("some_limit", 4)
        ):
            return self.data["some"]

        return self.data["many"]


class PluralizationParsingRule(ParsingRule):

    def matches(self, data: dict) -> bool:
        if "one" in data and "many" in data:
            return True
        return False

    def parsed_value(self, data: dict) -> Parsable:
        return PluralizedParsable(data)
